/*
   History:
    Started: Aug 9 by Lawrence Foard (entropy@world.std.com), to allow user 
     process control of SCSI devices.
    Development Sponsored by Killy Corp. NY NY
*       Copyright (C) 1992 Lawrence Foard

    This version has been ported to the lk 2.4 series kernels by
    D. Gilbert 2001/01/22. Version 1.5.1
*       Copyright (C) 2001 Douglas Gilbert
    It is recommended that the sg version 3 driver that is provided with
    the lk 2.4 series is used rather than this one. This driver is for
    those who need some of the numerous idiosyncracies of old driver which
    has been in Linux since 1992. This driver was part of the standard
    kernel prior to lk 2.2.6 (i.e. until April 1999) .
    Additions:
     - target_status, host_status, driver_status in sg_header structure,
       as per sg version definition
     - SG_GET_VERSION_NUM ioctl yields 1.5.<n>
*/

#ifndef _SCSI_GENERIC_H
#define _SCSI_GENERIC_H

/* 
 An SG device is accessed by writing "packets" to it, the replies
 are then read using the read call. The same header is used for 
 reply, just ignore reply_len field.
*/

struct sg_header
 {
  int pack_len;    /* length of incoming packet <4096 (including header) */
  int reply_len;   /* maximum length <4096 of expected reply */
  int pack_id;     /* id number of packet */
  int result;      /* 0==ok, otherwise refer to errno codes */
  unsigned int twelve_byte:1; /* Force 12 byte command length for group 6 & 7 commands  */
  unsigned int target_status:5;   /* [o] scsi status from target */
  unsigned int host_status:8;     /* [o] host status (see "DID" codes) */
  unsigned int driver_status:8;   /* [o] driver status+suggestion */
  unsigned int other_flags:10;    /* unused */
  unsigned char sense_buffer[16]; /* used only by reads */
  /* command follows then data for command */
 };

/* ioctl's */
#define SG_SET_TIMEOUT 0x2201  /* set timeout *(int *)arg==timeout */
#define SG_GET_TIMEOUT 0x2202  /* get timeout return timeout */

#define SG_EMULATED_HOST 0x2203 /* true for emulated host adapter (ATAPI) */

/* Used to configure SCSI command transformation layer for ATAPI devices */
#define SG_SET_TRANSFORM 0x2204
#define SG_GET_TRANSFORM 0x2205

#define SG_GET_VERSION_NUM 0x2282 /* Example: version 1.5.1 yields 10501 */

#define SG_DEFAULT_TIMEOUT (60*HZ) /* 1 minute timeout */
#define SG_DEFAULT_RETRIES 1

#define SG_MAX_QUEUE 4 /* maximum outstanding request, arbitrary, may be
			  changed if sufficient DMA buffer room available */

#define SG_BIG_BUFF 32768

#endif
